#!/bin/bash
#
# Template for install.sh

#Do not change
SCRIPT_VERSION="2.01"

#######################################
#######################################
#
# Main install() function to be
# modified for custom installs.
# This scriptcan only be run on builds
# 3.16/4.16 and higher
#
#######################################
#######################################
install()
{

# These 4 Variables are read by check_versions()
# Change them or empty them as needed.
# At a minimum MAX_4_MINOR_VERSION or MAX_3_MINOR_VERSION
# must be set if check_versions() is going to be called.
#
MIN_3_MINOR_VERSION="25"
MAX_3_MINOR_VERSION="28"
MIN_4_MINOR_VERSION="25"
MAX_4_MINOR_VERSION="28"
check_versions
if [ $? = 0 ]; then
    echo -n
    #uncomment exit if script is version protexccted
    #exit 1
fi


#test for SN product line, exits on failure
require_sn

#test for SN product line, returns 0 on success, 1 on failure
check_sn


#test for g25 cpu, exits if no match
#require_cpu_g25

#test for g25 cpu, returns 0 on success, 1 on failure
#check_cpu_g25

#test for bt6k cpu, exits if no match
#require_cpu_bt6k

#test for bt6k cpu, returns 0 on success, 1 on failure
#check_cpu_bt6k

#Unzip everything to / but install.sh
unzip_file "$1"

exit 0


} #end of install(), do not remove


#######################################
#######################################
##
##  Core Functions andtests below
##  DO NOT MODIFY CODE BELOW THIS LINE
##  Add custom code to the install()
##  function above
##
#######################################
#######################################


if [ ! -e "/etc/version" ]; then
 echo "Unit does not appear to be a Redlion device, exiting."
 exit 1
fi

eval . /etc/version &>/dev/null
if [ "$?" != "0" ]; then
   echo "Build version is older than 3.16/4.16, cannot continue"
   exit 1
fi


printVer()
{
    local VERSION="unknown"
    if [ -n "$BUILD_MINORVER" -a -n "$BUILD_MAJORVER" ]; then
        VERSION="$BUILD_MAJORVER.$BUILD_MINORVER"
    fi
    echo "$VERSION"
}

print_version()
{
    printVer
}


#Global vars
CPUFILE="/proc/cpuinfo"
BT6KCPU="AT91SAM9260"
G25CPU="AT91SAM9X5"

# function checkCPU
# valid cpu names are :
#
# g25 bt6k
# returns 0 on match
# returns 1 if no match
#
checkCPU()
{
   CPU="$1"
   ERROR=1

   if [ "$CPU" = "g25" ]; then
      grep -q "$G25CPU" $CPUFILE &>/dev/null
      if [ "$?" = "0" ]; then
         echo "CPU is a Red Lion g25 ($G25CPU)"
         ERROR=0
      fi
   elif [ "$CPU" = "bt6k" ]; then
      grep -q "$BT6KCPU" $CPUFILE &>/dev/null
      if [ "$?" = "0" ]; then
         echo "CPU is a Red Lion bt6k ($BT6KCPU)"
         ERROR=0
      fi
   fi

   return $ERROR
}


# function check_cpu_bt6k
#
# check if CPU is a bt6k CPU9, exit on failure
#
#
check_cpu_bt6k()
{
   local RETURN=1
   grep -q "$BT6KCPU" $CPUFILE &>/dev/null
   if [ "$?" = "0" ]; then
      RETURN=0
   fi
   return $RETURN

}

# function require_cpu_bt6k
#
# check if CPU is a bt6k CPU9
#
# returns 0 on match
# returns 1 if no match
#
require_cpu_bt6k()
{
   check_cpu_bt6k
   if [ "$?" = "0" ]; then
      echo "CPU is a Red Lion bt6k ($BT6KCPU)"
   else
      echo "CPU is not a Red Lion bt6k ($BT6KCPU), exiting"
      exit 1
   fi
}

# function check_cpu_g25
#
# check if CPU is a G25
#
# returns 0 on match
# returns 1 if no match
#
check_cpu_g25()
{
   local RETURN=1
   grep -q "$G25CPU" $CPUFILE &>/dev/null
   return $?
   if [ "$?" = "0" ]; then
      RETURN=0
   fi
   return $RETURN
}

# function require_cpu_g25
#
# check if CPU is a G25, exit on failure
#
require_cpu_g25()
{
   check_cpu_g25
   if [ "$?" = "0" ]; then
      echo "CPU is a Red Lion g25 ($G25CPU)"
   else
      echo "CPU is not a Red Lion g25 ($G25CPU), exiting"
      exit 1
   fi
}

#function checkCPUs
#
# Check a list of CPU types
# against checkCPU
#
# returns 0 on match
# returns 1 if no match (error)
#
checkCPUs()
{
   local CPUS="$@"
   local ERROR=1
   for cpu in $CPUS; do
      checkCPU "$cpu"
      if [ "$?" = "1" ]; then
         ERROR=0
         break
      fi
   done

   return $ERROR
}


#function check_sn
#
# Check if the device is in the SN product line
# bt6k, g25
#
# returns 0 on match
# returns 1 if no match (error)
#
check_sn()
{
   checkCPUs "g25 bt6k"
   return $?
}

#function require_sn
#
# Check if the device is in the SN product line
# bt6k, g25
#
require_sn()
{
    check_sn "g25 bt6k"
    if [ $? != 0 ]; then
       echo "Error: This package is for Red Lion SN devices only"
       exit 1
    fi
}



#function verComp
#
# compare 2 versions
#
# Return 0 on success
# Return 1 when ver1 > ver2
# Return 2 when ver1 < ver2
#
vercomp () {
   if [[ $1 == $2 ]]; then
      return 0
   fi
   local IFS=.
   local i ver1=($1) ver2=($2)
   # fill empty fields in ver1 with zeros
   for ((i=${#ver1[@]}; i<${#ver2[@]}; i++)); do
      ver1[i]=0
   done
   for ((i=0; i<${#ver1[@]}; i++)); do
      if [[ -z ${ver2[i]} ]]; then
         # fill empty fields in ver2 with zeros
         ver2[i]=0
      fi
      if ((10#${ver1[i]} > 10#${ver2[i]})); then
         return 1
      fi
         if ((10#${ver1[i]} < 10#${ver2[i]})); then
         return 2
      fi
   done
   return 0
}



# function checkVersion
# Version checking rules
#
#  Arg1 - max version
#       - less than or equal to
#  Arg2 - min version (optional)
#       - greater than or equal to
#
# Return 0 - match is ok
# Return 1 - match is out of range (error)
#
# Examples :
#
#  checkVersion "4.23"
#    - will return 0 on 4.23 and previous
#    - will return 1 on others
#
#  checkVersion "4.23" "4.21"
#    - will return 0 on 4.23, 4.22 and 4.21
#    - will return 1 on 4.20 and below, and 4.24 and higher
#
checkVersion()
{
   if [ -z "$BUILD_MINORVER" ]; then
      echo "Error:  Cannot check version, BUILD_MINORVER is empty"
      return 1
   fi

   if [ -z "$BUILD_MAJORVER" ]; then
      echo "Error:  Cannot check version, BUILD_MAJORVER is empty"
      return 1
   fi

   local VERSION="$BUILD_MAJORVER.$BUILD_MINORVER"
   local HIGH_VERSION=$1
   local LOW_VERSION=$2
   local ERROR=1
   local RETCODE=0

   if [ -z "$HIGH_VERSION" ]; then
      echo "Error:  Cannot check version, no comparison given"
      return $ERROR
   else
      vercomp "$VERSION" "$HIGH_VERSION"
      RETCODE=$?
      if [ $RETCODE = 0 ]; then
         echo "current $VERSION = check $HIGH_VERSION, OK!"
         ERROR=0
      elif [ $RETCODE = 1 ]; then
         echo "current $VERSION > check $HIGH_VERSION, BAD!"
         ERROR=1
      elif [ $RETCODE = 2 ]; then
         echo "current $VERSION < check $HIGH_VERSION, OK!"
         ERROR=0
      fi

      if [ "$ERROR" = "0" ]; then
         if [ -n "$LOW_VERSION" ]; then
            vercomp "$LOW_VERSION" "$VERSION"
            RETCODE=$?
            if [ $RETCODE = 0 ]; then
               echo "check $LOW_VERSION = current $VERSION, OK!"
               ERROR=0
            elif [ $RETCODE = 1 ]; then
               echo "check $LOW_VERSION > current $VERSION, BAD!"
               ERROR=1
            elif [ $RETCODE = 2 ]; then
               echo "check $LOW_VERSION < current $VERSION, OK!"
               ERROR=0
            fi

         fi
      fi
   fi

   #if [ "$ERROR" = "1" ];
   #   echo -n "Error: Current build version $VERSION,  Package is for $HIGH_VERSION"
   #   if [ -n "$LOW_VERSION" ];
   #      echo -n "-$LOW_VERSION"
   #   else
   #      echo -n " and below"
   #   fi
   #   echo
   #fi

   return $ERROR

}

#Unzip everything to / but install.sh
#
# Return 0 on success
# Return 1 on error
#
unzip_file()
{
    local RETURN=0
    #ZIP file archive must be called:
    local FILE_NAME=$1

    if [ ! -e "$FILE_NAME" ]; then
        echo "Cannot find package file $FILE_NAME"
        exit 1
    fi

    #unzip everything but the script and special dirs
    unzip -o $FILE_NAME -x "install.sh" -d /

    if [ $? != 0 ]; then
        RETURN=1
    fi

    return $RETURN
}

check_versions()
{
    local VALID=0

    if [ -z "$MAX_3_MINOR_VERSION" -a -z "$MAX_4_MINOR_VERSION" ]; then
        echo "No version set to check, cannot validate build version"
        exit 1
    fi

    if [ -n "$MAX_3_MINOR_VERSION" -a -n "$MIN_3_MINOR_VERSION" ]; then
        checkVersion "3.$MAX_3_MINOR_VERSION" "3.$MIN_3_MINOR_VERSION"
        if [ $? = 0 ]; then
            VALID=1
        fi
    elif [ -n "$MAX_3_MINOR_VERSION" ]; then
        checkVersion "3.$MAX_3_MINOR_VERSION"
        if [ $? = 0 ]; then
            VALID=1
        fi
    fi

    if [ -n "$MAX_4_MINOR_VERSION" -a -n "$MIN_4_MINOR_VERSION" ]; then
        checkVersion "4.$MAX_4_MINOR_VERSION" "4.$MIN_4_MINOR_VERSION"
        if [ $? = 0 ]; then
            VALID=1
        fi
    elif [ -n "$MAX_4_MINOR_VERSION" ]; then
        checkVersion "4.$MAX_4_MINOR_VERSION"
        if [ $? = 0 ]; then
            VALID=1
        fi
    fi

    if [ $VALID = 0 ]; then
        local THIS_VER=$(printVer)
        if [ $? = 1 ]; then
            THIS_VER=""
        fi
        echo "This package is not valid for this build $THIS_VER"
    fi

    return $VALID

}

install "$@"
exit 0

#######################################
##  DO NOT MODIFY ABOVE/BELOW THIS LINE
##  Except for in the install() function
##  at the top
#######################################
