#!/bin/bash

mkdir -p accessory

if [ ! -e Makefile ]; then
	cp defaults/Makefile .

	echo "Where is your toolchain installed? (path to wsbu.mk file)"
	read toolpath
	echo

	if [ -f "$toolpath" ]; then
		toolpath="$(dirname "$toolpath")"
	fi

	# Get absolute path
	toolpath="$(cd "$toolpath"; pwd)"
	if [ -f "$toolpath/wsbu.mk" ]; then
		sed -i "s;^#\?include .*$;include $toolpath/wsbu.mk;" "Makefile"
	else
		echo "Did not find wsbu.mk. Please update the \"include\" line in"
		echo "    Makefile"
	fi
fi

if [ ! -e accessory/sdk.init ]; then
	cp defaults/sdk.init accessory
fi

if [ ! -e accessory/sdk.conf ]; then
	cp defaults/sdk.conf accessory
fi

if [ ! -e accessory/install.sh ]; then
	cp defaults/install.sh accessory
fi

echo "WARNING: Only say yes to the following options once you have tested your program"
echo "on a device and are sure it does what it is supposed to"
echo "(You can run this configure script later to change your options)"
echo "Do you want this program to run when the device boots? (y/N)"
read bootStart
echo
case "$bootStart" in
	YES|Yes|yes|Y|y)
		sed -i s/\#chkconfig/chkconfig/ accessory/install.sh
		echo "Application will run at boot time."
		;;
	*)
		echo "Application will not run at boot time."
		;;
esac

echo
echo "Do you want this program to persist through reflashing the device? (y/N)"
read persist
echo
case "$persist" in
	YES|Yes|yes|Y|y)
		sed -i s/\#cp/cp/ accessory/install.sh
		echo "Application will be installed after a reflash"
		;;
	*)
		echo "Application will be erased during a reflash"
		;;
esac
exit 0
