# IOG HTTP Rest API

## Set Configuration

Subsystems can be configured (write-only) using the "gau" cgi through the device web server. This can be done using a system call to curl

	curl --digest-u admin:123456 --data 'subsystem=sshserver&enable=y&gracetime=90&action=Apply' 192.168.0.1:10000/cgi-bin/gau
	    --digest                       - use digest authentication *New in 4.26
	    -u admin:123456                - authenticate using user "admin" and password "123456"
	    --data '...'                   - URL-encoded string of key/value pairs to specify configuration
	        subsystem=sshserver        - which subsystem settings apply to
	        enable=y&gracetime=90      - subsystem-specific options
	        action=Apply               - Whether to "Apply" settings now or just "Save" them for later
	    192.168.0.1:10000/cgi-bin/gau  - IP of device, at standard HTTP port 10000, using gau cgi

## Get Configuration

Read is handled by a separate cgi: cfgfetch.cgi, where multiple subsystems can be fetched at once on a per-value basis.

	curl --digest -u admin:123456 --data 'sshEnable=/GAUCfg/sshserver/enable/text()&gpsTty=/GAUCfg/gps/ttyport/text()' 192.168.0.1:10000/cgi-bin/cfgfetch.cgi
	    --digest                       - use digest authentication *New in 4.26
	    -u admin:123456                         - authenticate using user "admin" and password "123456"
	    --data '...'                            - URL-encoded string of key/value pairs to specify which config options to fetch
	        sshEnable                           - Custom key that will appear in json response
	        /GAUCfg/sshserver/enable/text()     - fetch the value of sshserver's "enable" config option
	    192.168.0.1:10000/cgi-bin/cfgfetch.cgi  - IP of device, at standard HTTP port 10000, using gau cgi

The response will be a json object containing a "data" attribute map of your keys populated with config values. The response for the above call may be:

	{"error" : false, "data" : {"gpsTty" : "gpsd","sshEnable" : "y"}}

## Package Installation

Packages can be uploaded and installed using curl by sending a form-data request
to the gau cgi. Example:

	curl --digest -u admin:123456 -F subsystem=package -F action=Install -F json=true -F "file=@/path/to/snpatch.zip" http://192.168.0.1:10000/cgi-bin/gau

> Change `/path/to/snpatch.zip` to be an absolute-path to your zip package.

A successful response will look like:

	{"error" : false, "message" : "Operation Succeeded."}
