# RAM Tags Infrastructure

Tags offer easier access to IODB registers through named points. Tags require a
name and IODB type/address coordinates, then optionally specify other attributes
like source, class and description.

* [Source](#markdown-header-source)
* [Fields](#markdown-header-fields)
    * [IODB Types](#markdown-header-iodb-types)
    * [Data Type](#markdown-header-data-type)
        * [Examples](#markdown-header-examples)
    * [Class](#markdown-header-class)
    * [Deadband](#markdown-header-deadband)

## Source

`/etc/iog/tags.csv` is the source of truth for tag definitons. This file should
be considered **read only** to avoid conflict with other subsystems.

## Fields

The following table lists all tag properties in the order they appear in the csv
file

Index | Label | Description
:----:|-------|------------
0     | Tag Name | Name of the tag
1     | Station Name |
2     | Station Number |
3     | I/O Type # | Numeric type (see below)
4     | I/O Register # | Register address (0-indexed)
5     | Data Type | Key for interpreting this (and surrounding) registers
6     | Actual Min |
7     | Actual Max |
8     | Scaled Min |
9     | Scaled Max |
10    | Engineering Units |
11    | Decimals |
12    | Off Message |
13    | On Message |
14    | Description | Long-form description
15    | Retain | Preserve across reboot
16    | Source | 1 = system (consider it "read only"), or 0
17    | Class | Typically "model", "status", or "user"
18    | Deadband | Range of change to ignore

### IODB Types

These numeric types are used by the internal `IODBRead` and `IODBWrite`
functions.

|||
|-:|-|
| 0  | Analog Input |
| 1  | Analog Output |
| 10 | Discrete Input |
| 11 | Discrete Output |
| 20 | Long Input |
| 21 | Long Output |
| 22 | Float Input |
| 23 | Float Output |

### Data Type

The data type is a concatenation of flags specifying how to interpret an
**analog** type register. Positional argument can be used to include surrounding
registers in largert/smaller values.

Registers values will be interpreted according to this data type when referenced
by tag on the Web UI Test I/O page, as well as when logged to disk and sent to
the cloud through our RamQTT solution.

The basic form is 1A8X, where

| | Name | Required | Default | Detail |
|-|------|----------|---------|--------|
| 1 | Index || 0 | 0-based index of reguister in series |
| A | Type | \* | A | Character key for type interpretation. Options: **F**loat, **I**integer, **A**scii character, **B**it. |
| 8 | Bits | \* | \* | Bits used in value. Default is 16 for integer, 32 for float, 1 for bit, 2 for ascii |
| X | Extra || UL | Additional flags for interpretation. **U**n/**S**igned, **B**ig/**L**ittle endian, **P<precision>** |

#### Examples

|||
|-|-|
|| 16-bit unsigned integer, little endian |
| I16 | 16-bit unsigned integer, little endian |
| 0I16 | 16-bit unsigned integer, little endian |
| I16U | 16-bit unsigned integer, little endian |
| 0I16U | 16-bit unsigned integer, little endian |
| 1F32 | Second half of a 32-bit floating point number |
| 0A4 | First two characters of a 4-char ascii string |
| 2B1 | Bit, stored in the 3rd bit location of a bit-packed register |

### Class

Tag definitions are split into several categories, indicating usage/intent and
access restrictions.

**status** tags are used for system stats. These are completely immutable, and
any imported tags with tag names or coordinates defined in this range will be
ignored. These tags should be considered **read only** as they will be
overwritten by the system.

**model** tags are used for model-specific tags like hardware I/O. These can be
renamed to better describe attached hardware, but the coordinates cannot be
changed.

All other tags will be assigned the **user** class by default.

### Deadband

This is an absolute value applied as +/- window around the last recorded value.
Assumed if missing. Valid ranges are 0-32768, and decimals are allowed for float
types.
