// 2015 Red Lion Controls, Inc. All rights reserved. Red Lion, the Red Lion logo and Sixnet are registered trademarks of Red Lion Controls, Inc. All other company and product names are trademarks of their respective owners.
/*


 ******************************************************************************************************
 |  WHEN    | WHO | WHAT                                                                              |
 |----------------------------------------------------------------------------------------------------|
 | 10/23/15  | bet | file created                                                                      |
 |----------------------------------------------------------------------------------------------------|
 |          |     |                                                                                   |
 |----------------------------------------------------------------------------------------------------|
 |          |     |                                                                                   |
 ******************************************************************************************************


Example Usage :

    #include "file_imonitor.hpp"
    #define CONFIG_FILE "/etc/configfile.conf"
    cFileMonitor * l_Cp_MonitorConfigFile = new cFileMonitor();
    //l_Cp_MonitorConfigFile->SetDebug(1); //for debugging
    //FILE FLAGS are optional.  If omitted, or a 0 is passed in then the defaults (below) will be sued in the class
    unsigned int l_ui_flags = (IN_ATTRIB | IN_CREATE | IN_MODIFY | IN_CLOSE_WRITE | IN_DELETE_SELF | IN_MOVE_SELF);
    l_Cp_MonitorConfigFile->imonitorInit((char *)CONFIG_FILE, l_ui_flags);


    //File states:
    //      (start state)                            (thread changes)                            (controller changes)
    //   eIMONITOR_FILE_PROCESSED  -->  eIMONITOR_FILE_CHANGED/eIMONITOR_FILE_REMOVED   -->    eIMONITOR_FILE_PROCESSING _
    //             ^                                                                                                      |
    //             |______________________________________________________________________________________________________|
    //

    while (1)
    {
        if ( eIMONITOR_FILE_CHANGED == l_Cp_MonitorConfigFile->GetFileStatus() )
        {
            l_Cp_MonitorConfigFile->ProcessingFile(true); //Notify thread we are processing the file
            fprintf(stderr, "I see the file has changed\n");
            sleep(5);
            fprintf(stderr, "Done processing file\n"); //Notify thread we are done processing the file
            l_Cp_MonitorConfigFile->ProcessingFile(false);
        }
        if ( eIMONITOR_FILE_REMOVED == l_Cp_MonitorConfigFile->GetFileStatus() )
        {
            //Can't proccess file, wait for it to show up
            l_Cp_MonitorConfigFile->ProcessingFile(false);
            fprintf(stderr, "I see the file has been removed\n");
            sleep(5);
        }
        sleep(1);
    }

 */

#ifndef __MONITOR_CLASS_HPP
 #define __MONITOR_CLASS_HPP

//using namespace std;
//#include <string>

#include <sys/types.h>
#include <stdint.h>


typedef enum
{
    eIMONITOR_FILE_PROCESSED,
    eIMONITOR_FILE_PROCESSING,
    eIMONITOR_FILE_CHANGED,
    eIMONITOR_FILE_REMOVED,
} eFileStatus;


#define MAX_FILE_LEN 512

class cFileMonitor
{

public:

    int  imonitorInit(char * l_ucp_filename, int8_t l_i8_flags=0);
    eFileStatus  GetFileStatus(void);
    void SetDebug(unsigned int l_ui_debug);
    void StopMonitor(void);
    void ProcessingFile(bool l_b_status);


    /*  Needed public by pthread_create */
    static void * inotifyMonitor_helper(void *context);

    ~cFileMonitor();      //destructor
    cFileMonitor();       //default constructor


protected:

    /* watcher and file descriptor */
    int C_i_watch_wd;
    int C_i_watch_fd;
    int c_ui_default_flags;

    /*  Hold the filename to be monitored */
    unsigned char C_uca_filename[MAX_FILE_LEN+1];

    /* true once the thread is created */
    bool C_thread_created;

    /* debug mode - more syslog messages */
    unsigned int C_ui_debugMode;

    /*  File watch ioctl flags */
    uint8_t C_ui8_file_watch_flags;

    /* Inotify vars for file change notifications  */
    eFileStatus volatile C_file_status;

    /* default false, if the user calls obj->StopMonitor() it goes to true, and the thread exits */
    uint8_t volatile C_file_stop_monitor;


    /*  Main thread file watcher */
    void * inotifyMonitor(void);

    bool  inotifyWatchAdd(void);
    void  changeAction(void);
    void  removeAction(void);
    eFileStatus   inotifyEventProcess(void);
    void  WarnAddError(void);


    cFileMonitor(cFileMonitor &); //Copy Construct`or


private:


};


 #endif
