/*


 c2017 Red Lion Controls, Inc. All rights reserved. Red Lion, the Red Lion logo and Sixnet
 are registered trademarks of Red Lion Controls, Inc. All other company and product
 names are trademarks of their respective owners.

 ******************************************************************************************************
 |  WHEN    | WHO | WHAT                                                                              |
 |----------------------------------------------------------------------------------------------------|
 | 8/15/07  | bet | file created                                                                      |
 |----------------------------------------------------------------------------------------------------|
 |          |     |                                                                                   |
 ******************************************************************************************************

 */

#ifndef __LOGGER_HPP
 #define __LOGGER_HPP

#include <string>
using namespace std;

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>

#define DEFAULT_LOG_DATE_FORMAT "%m/%d %H:%M:%S"
#define MAX_FILE_NUM      1000
#define MAX_STRING_LEN    256
#define DOUBLE_STRING_LEN 512
#define MAX_DATALOG_LEN   8192
#define MAX_HEADER_SIZE   65536
#define TIMESTAMP_MIN 19700101000000LL  // consistent with Epoch
#define TIMESTAMP_MAX 20991231235959LL
#define CMD_INJCT_CHAR_STR ";()|&`"
#define DATALOG_EMAIL_DIR "/tmp/datalog_email/"
#define DATALOG_UPLOAD_DIR "/tmp/fileupload/"
#define ROTATE_NUMBER_DEFAULT 1
#define ROTATE_NUMBER_MIN 1
#define ROTATE_NUMBER_MAX 10

enum DELIVERY_MODE  {
    SEND_BY_NONE = 0,
    SEND_BY_FTP,
    SEND_BY_EMAIL,
    SEND_BY_BOTH
};

enum FTP_SSL_MODE  {
    NON_SSL = 0,
    IMPLICIT_SSL,
    EXPLICIT_SSL
};


struct FtpUpload
{
    char        sslmode;
    bool        ftpactivemode;
    int         port;
    string*     server;
    string*     path;
    string*     usrname;
    string*     pswd;
};

class cLogger
{

public:



    ~cLogger();      //destructor
    cLogger();       //default constructor

    /*
    Aftter new[] call this function to initialize the logger :

         p->LogInit(l_s_LogfileName, l_i_KBtoRotate, l_b_PrePentTimeDate, l_cp_TimeDateFormat, l_i_rotate_number )

        l_s_LogfileName      : Logfile /path/to/logfilename to log entries to when p->LogIt("log this") is called
        l_i_KBtoRotate       : rotate the log file after it is >= this int size.  Size is in KB and NOT bytes ( 50 - 2048 ).
                               Note: Every log rotates!
        l_b_PrePentTimeDate  : set to true if you want each line to prepent with a "date/time: " format, default is in DEFAULT_LOG_DATE_FORMAT
        l_cp_TimeDateFormat  : do not pass if you want to use the default in l_cp_TimeDateFormat . Otherwise use the format spefified in strftime()
        l_i_rotate_number    : Default 1, Min 1, Max 10 - num of files to keep in the rotation
    */
    bool LogInit(string, int, bool=true, char * =NULL, int =ROTATE_NUMBER_DEFAULT);

    //call to force a log to rotate
    virtual void Rotate(void);

    /*
      Function to call when you want something logged.
    */
    virtual bool LogIt(string, bool=false);

    //debug level for cLogger to syslog. 5 is very noisy
    void SetDebug(int);

    //send log messages to the syslog as well as to the set log
    void AlsoToSyslog(bool);

    // "last line repeated N time(s)", default is false (off)
    void SuppressLastMessage(bool);

    // Value in KB, can be changed after the log is created
    int c_i_MaxLogFileSize;

    //over ride (ignore) timestamp for the next message to be inserted, if timestamping is on
    void NoTimestampThisMessage(void);

    //Don't strip ^M from message
    void DontModifyThisMessage(void);

protected:

    string c_s_LogFileName;

    int  c_i_LogFH;
    bool c_b_AddDateTime; //shouldnt be able to change this after new()
    int c_i_rotate_number;

    int c_b_Debug;
    string c_s_LastLogMessage;
    bool c_b_SupressLastMesg;
    bool c_b_AlsoLogToSyslog;
    string c_s_DateFormat;
    int c_i_LastMessageCounter;
    bool c_b_IgnoreThisDateTime;
    bool c_b_DontModifyThisMessage;

    virtual bool OpenLogFile(void);
    void CloseLogFile(void);
    void CheckRotate(void);


    cLogger(cLogger &); //Copy Constructor


private:


};

class cDatalogger : public cLogger
{

public:
    ~cDatalogger();      //destructor
    cDatalogger();       //default constructor

    char c_cp_filenamemajor[DOUBLE_STRING_LEN+1];
    string c_s_filesuffix;
    bool Init(struct LogInitCfg *);

    virtual void Rotate(void);

    bool LogData( void*, int );
    bool LogData( char* );

protected:
    string c_s_datalogname;
    string c_s_logheader;
    string c_s_fileprefix;
    string c_s_timestampfmt;
    string c_s_zippaswd;
    string c_s_emailrecipient;
    struct FtpUpload c_st_ftpupload;
    char c_c_deliverymode;
    bool c_b_ftpactivemode;
    long long * c_llp_stamp;
    int c_i_filecnt;
    int c_i_maxfilenum;
    bool c_b_zipflag;
    bool c_b_newfileflag;
    pthread_t c_th_wrapperthread;

    friend void* ThreadFileWrapper(void*);

    //virtual void CheckRotate(void);
    void ShiftFileStamps(void);
    bool StartFileWrapperThread(void);
    void LogHeader(void);
    void* ziplogfile(void*);
    bool IsStampValid(long long);

private:


};

void* ThreadFileWrapper(void*);

struct LogInitCfg
{
    string*     filepath;
    string*     fileprefix;
    string*     filesuffix;  //e.g ".csv" or ".txt" or empty
    int         filesizeK;
    int         maxfilenum;
    string*     timestampfmt;
    string*     logheader;      // empty for no log header
    bool        zipflag;
    bool        newlogflag;
    string*     zippaswd;
    char        deliverymode;
    string*     emailrecipient;
    struct FtpUpload ftpupload;

};


 #endif


