// ©2015 Red Lion Controls, Inc. All rights reserved. Red Lion, the Red Lion logo and Sixnet are registered trademarks of Red Lion Controls, Inc. All other company and product names are trademarks of their respective owners.
/* $Workfile: timer.hpp $  $Date: 2/12/02 4:42p $



                              REVISION HISTORY

ͻ
   WHEN    WHO                             WHAT                           
͹
 11/30/00  tkb  Created to ease clock timing routines                     
Ķ
 12/01/00  tkb  Added IsTimerRunning to check if a timer has stopped...   
Ķ
 12/08/00  tkb  Added GetElapsedTime() to get feedback on time values...  
Ķ
 03/27/01  tkb  Added GetFormattedElapsedTime().  Fixed GW50 issues.      
Ķ
                                                                          
ͼ  */

// !!!!!!!!!!!!!!!  SEE TIMER.CPP file for detailed instructions !!!!!!!!!!!!!!


#ifndef  TIMER_HPP
    #define TIMER_HPP

    #ifdef OS2
        #include "os2defs.h"
        #include <time.h>
        #include <sys\timeb.h>
        #include <sys\types.h>
    #endif

//    #ifdef PLAT_LINUX
        #include <time.h>
        #include <sys/time.h>
        #include <unistd.h>
//    #endif

    #ifdef PLAT_GW50SERIES
        #include "time.h"
        #include "net.h"
        #include "support.h"
    #endif

    #include <stdlib.h>
    #include <stdio.h>




/*--------------------------- Class cTimer ----------------------------------------------------------------*/

class cTimer
{
public:                                                                                             /*
       ------------------ VARIABLES ----------------------------------------------------------------*/

    enum eTIMERTYPE
    {
        eSECOND,
        eMILLISECOND
    };
    struct c_S_timer // need to keep track of seconds and milliseconds
    {
        unsigned long c_l_seconds;
        unsigned long c_l_milliseconds;
    };
    /*
------------------ FUNCTIONS ----------------------------------------------------------------*/

    // Base functions:

    cTimer(void);                // Default Constructor
    cTimer(unsigned long);               // Second-Timeout Constructor
    cTimer(eTIMERTYPE);          // Tick Type Constructor
    cTimer(eTIMERTYPE, unsigned long);   // Full Constructor

    ~cTimer(void);  // Destructor.

    void Start(void);  //start with previous Timeout value
    void Start(unsigned long);  //start with new Timeout value

    void Stop(void);

    bool IsTimerDone(void);

    bool IsTimerRunning(void);

    bool IsTimerEnabled(void);

    unsigned long GetElapsedTime(void);

    char * GetFormattedElapsedTime(void);

    void SetTimeout(unsigned long);

    unsigned long GetTimeout(void);

protected:                                                                                           /*
          ------------------ VARIABLES --------------------------------------------------------------*/

    unsigned long c_ul_timeoutValue;

    char c_A_displayBuffer[80];
    /*

------------------ FUNCTIONS --------------------------------------------------------------*/


    void GetTime(c_S_timer &);

private:                                                                                             /*
        ------------------ VARIABLES ----------------------------------------------------------------*/

    c_S_timer c_T_startTime;         //All values stored as MILLISECONDS
    c_S_timer c_T_now;               // "    "       "    "     "


    bool c_b_TimerActive;
    unsigned long c_ul_multiplier;

    /*
------------------ FUNCTIONS ----------------------------------------------------------------*/

    // Base functions:

    cTimer(cTimer &);  // Use of Copy constructor is NOT allowed.

    void operator = (cTimer &);    // Use of assignment operator is NOT allowed.


};



/*
To use the Timer class, create a timer as a second or millisecond timer:
You have 4 ways of creating a timer:


+   cTimer l_T_timer;                //Default type: SECOND, Default timeout=1
+   cTimer l_T_timer(5);             //Default type: SECOND, timeout=value (5)
+   cTimer l_T_timer(eMILLISECOND);  //Type: MILLISECOND/SECOND,  Default timeout=1
+   cTimer l_T_timer(eSECOND,5);     //Type: MILLISECOND/SECOND,  timeout=value (5)


Then set the timeout value (if needed ... can be changed anytime during execution):

+   l_T_timer.SetTimeout(5);  //sets 5 second timeout


Start the timer (with or without a new timeout value):

+   l_T_timer.Start();   //is like time(&l_T_then)
+   l_T_timer.Start(5);


Check the timer for expiration:

+   if(true == l_T_timer.IsTimerDone())
+       {
+       //Timeout has occured... just like
+       //if (5 <= (unsigned long)abs((int)difftime(c_T_now, c_T_then)))
+
+       //...do your work...
+
+       l_T_timer.Stop();   //Without this, IsTimerDone will keep saying true
+                           //which could cause you to repeat this loop.
+       }


Check the timer while running:

+   if(true == l_T_timer.IsTimerRunning())
+       {
+       //... just like
+       //... if (5 > (unsigned long)abs((int)difftime(c_T_now, c_T_then)))
+       // This is useful to do work DURING the timer 'countdown'.
+       // This will return a false if Start() has not been called,
+       // Stop() is called, or once the timeout has occurred (IsTimerDone()==true)
+       //...do your work...
+
+       }


Get actual elapsed time since start:

+   unsigned long l_value = l_T_timer.GetElapsedTime();
+         //Will return 0 if stopped, otherwise it will return the number of
+         //Seconds/Milliseconds based on the timer type at creation.
+         //This function is independant of the timeout value.  Start() Stop() only affect it.


Stopping the Timer:

+   l_T_timer.Stop();   //This will cause IsTimerDone to always report false
+                       // !!!Do this after handling the IsTimerDone == true case!!!


At any time you can get or set the timeout value:

+   l_T_timer.SetTimeout(l_value);
+   l_value = l_T_timer.GetTimeout();



          L O G I C    T A B L E
ͻ
                0         1         2         3         4         5         6         7         8  
      Timeline: 012345678901234567890123456789012345678901234567890123456789012345678901234567890  
                    *               *         *         *                                          
      Event*  : CreateTimer      Start()              Stop()                                       
 Response       (Sec , 10)                <timeout>                                                
  of:                                     <occurs >                                                
  
                                                                                                   
 IsTimerDone    ------------------------------++++++++++-----------------------------------------  
                                                                                                   
                                                                                                   
 IsTimerRunning --------------------++++++++++---------------------------------------------------  
                                                                                                   
                                                                                                   
 IsTimerEnabled --------------------+++++++++++++++++++++----------------------------------------  
                                                                                                   
                                              111111111120                                         
 GetElapsedTime 0000000000000000000001234567890123456789000000000000000000000000000000000000000000 
                                                                                                   
                                                                                                   
ͼ

*/




inline
void cTimer::SetTimeout(unsigned long l_timeout)
{
    /*************************************************************\
    | SetTimeout -                                                |
    |         Change the timeout trigger value at any time.       |
    |                                                             |
    |  Entry Parameters:  LONG value in SEC/MILLISEC based on     |
    |                                          eTIMERTYPE.        |
    |           Returns:  None.                                   |
    |                                                             |
    |  Globals affected:  None.                                   |
    |                                                             |
    \*************************************************************/

    c_ul_timeoutValue = l_timeout * c_ul_multiplier;
}


inline
unsigned long cTimer::GetTimeout(void)
{
    /*************************************************************\
    | GetTimeout -                                                |
    |         Peek at the current value for the timeout.          |
    |                                                             |
    |  Entry Parameters:  None.                                   |
    |                                                             |
    |           Returns:  LONG value in SEC/MILLISEC based on     |
    |                                          eTIMERTYPE.        |
    |  Globals affected:  None.                                   |
    |                                                             |
    \*************************************************************/

    return((unsigned long)c_ul_timeoutValue/c_ul_multiplier);
}


inline
void cTimer::Start(void)
{
    /*************************************************************\
    | Start -                                                     |
    |         Starts the timer (default or preset timeout)        |
    |                                                             |
    |  Entry Parameters:  None.                                   |
    |                                                             |
    |           Returns:  None.                                   |
    |                                                             |
    |  Globals affected:  None.                                   |
    |                                                             |
    \*************************************************************/

    c_b_TimerActive = true;
    GetTime(c_T_startTime);

}



inline
void cTimer::Start(unsigned long l_value)
{
    /*************************************************************\
    | Start -                                                     |
    |         Starts the timer with timeout.                      |
    |                                                             |
    |  Entry Parameters:  timeout value                           |
    |                                                             |
    |           Returns:  None.                                   |
    |                                                             |
    |  Globals affected:  None.                                   |
    |                                                             |
    \*************************************************************/

    SetTimeout(l_value);
    Start();
}


inline
void cTimer::Stop(void)
{
    /*************************************************************\
    | Stop  -                                                     |
    |         Stops the timer.                                    |
    |                                                             |
    |  Entry Parameters:  None.                                   |
    |                                                             |
    |           Returns:  None.                                   |
    |                                                             |
    |  Globals affected:  None.                                   |
    |                                                             |
    \*************************************************************/

    c_b_TimerActive = false;

}



#endif  // TIMER_HPP
